#pragma once

#include <amulet/utils/image.hpp>

#include <amulet/resource_pack/dll.hpp>

namespace Amulet {

AMULET_RESOURCE_PACK_EXPORT PIL::Image::Image get_missing_pack_icon();

} // namespace Amulet
